#include <zephyr/zephyr.h>
#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(reset_reason, 4);

#define NO_RESET_FLAG (0)
#define MAX_RESET_REASONS 17

typedef struct
{
    uint32_t flag;
    const char* field;
    bool is_error;
} ResetReason;

static const ResetReason reset_reasons[MAX_RESET_REASONS] = 
{
    {0x00000001, "RESETPIN", false},
    {0x00000002, "DOG0", true},
    {0x00000004, "CTRLAP", true},
    {0x00000008, "SREQ", false},
    {0x00000010, "LOCKUP", true},
    {0x00000020, "OFF", true},
    {0x00000040, "LPCOMP", true},
    {0x00000080, "DIF", true},
    {0x00010000, "LSREQ", true},
    {0x00020000, "LLOCKUP", true},
    {0x00040000, "LDOG", true},
    {0x00800000, "MFORCED", true},
    {0x01000000, "NFC", true},
    {0x02000000, "DOG1", true},
    {0x04000000, "VBUS", true},
    {0x08000000, "LCTRLAP", true}
};

static void log_reset_reasons(uint32_t reset_reason)
{
    bool reasons_found = false;
    for (int i = 0; i < MAX_RESET_REASONS; i++)
    {
        if (reset_reason & reset_reasons[i].flag)
        {
            LOG_INF("Reset reason: %s", reset_reasons[i].field);
            reasons_found = true;
        }
    }
    if (!reasons_found)
    {
        LOG_INF("No reset reasons detected");
    }
}

void check_reset_reason(void)
{
    uint32_t reset_reason = NRF_RESET->RESETREAS;
    if (reset_reason != NO_RESET_FLAG)
    {
        log_reset_reasons(reset_reason);
        // Check for error conditions
        for (int i = 0; i < MAX_RESET_REASONS; i++)
        {
            if ((reset_reason & reset_reasons[i].flag) && reset_reasons[i].is_error)
            {
                LOG_ERR("Critical reset detected: %s", reset_reasons[i].field);
            }
        }
        NRF_RESET->RESETREAS = reset_reason; // clear reset
    }
}